using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;


namespace EMPExtNav
{
	/// <summary>
	/// EMPExtNav - Visual Studio .NET like Navigation Control
	/// You can use and redistribute this Piece of Code under
	/// the GPL (GNU PUBLIC LICENSE). Use this Code, just do
	/// not delete or modify this Copyright.
	/// Copyright (c) 2003 Andreas Mallek - mallek@empora.de
	/// </summary>

	public class ExtNav : System.Windows.Forms.UserControl
	{
		private System.ComponentModel.Container components = null;

		private int items=0;
		private int active_item = -1;
		private int hover_item = -1;
		private static int MAX_ELEMENTS = 15;
		private string[] menuitems = new string[MAX_ELEMENTS];
		private System.Drawing.Point[] item_start = new System.Drawing.Point[MAX_ELEMENTS];
		private System.Drawing.Point[] item_end = new System.Drawing.Point[MAX_ELEMENTS];
		private System.IntPtr[] handles = new System.IntPtr[MAX_ELEMENTS];
			
		private Color ActiveBackColor = System.Drawing.SystemColors.Control;
		private Color ActiveForeColor = System.Drawing.Color.Black;
		private Color InactiveBackColor = System.Drawing.Color.DarkBlue;
		private Color InactiveForeColor = System.Drawing.SystemColors.Control;
		private Color CtlBackColor = System.Drawing.Color.Yellow;

		public System.Windows.Forms.Form mdiparent;

		public Color ABackColor
		{
			get { return ActiveBackColor; }
			set { ActiveBackColor = value; }
		}

		public Color AForeColor
		{
			get { return ActiveForeColor; }
			set { ActiveForeColor = value; }
		}

		public Color IBackColor
		{
			get { return InactiveBackColor; }
			set { InactiveBackColor = value; }
		}

		public Color IForeColor
		{
			get { return InactiveForeColor; }
			set { InactiveForeColor = value; }
		}

		public Color CTLBackColor
		{
			get { return CtlBackColor; }
			set 
			{ 
				CtlBackColor = value; 
				this.BackColor = value;
			}
		}

		public ExtNav()
		{
			InitializeComponent();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		private void InitializeComponent()
		{
			// 
			// ExtNav
			// 
			this.ABackColor = System.Drawing.SystemColors.Control;
			this.AForeColor = System.Drawing.Color.Black;
			this.BackColor = System.Drawing.Color.FromArgb(((System.Byte)(255)), ((System.Byte)(251)), ((System.Byte)(233)));
			this.CTLBackColor = System.Drawing.Color.FromArgb(((System.Byte)(255)), ((System.Byte)(251)), ((System.Byte)(233)));
			this.Dock = System.Windows.Forms.DockStyle.Top;
			this.IBackColor = System.Drawing.Color.FromArgb(((System.Byte)(255)), ((System.Byte)(251)), ((System.Byte)(233)));
			this.IForeColor = System.Drawing.Color.Gray;
			this.Location = new System.Drawing.Point(203, 0);
			this.Name = "ExtNav";
			this.Size = new System.Drawing.Size(765, 30);
			this.TabIndex = 0;
			this.Load += new System.EventHandler(this.ExtNav_Load);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.ExtNav_Paint);
			this.MouseMove += new System.Windows.Forms.MouseEventHandler(this.ExtNav_MouseMove);
			this.MouseLeave += new System.EventHandler(this.ExtNav_MouseLeave);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.ExtNav_MouseDown);

		}

		private void ExtNav_Load(object sender, System.EventArgs e)
		{
		}

		public void add_item(string text, System.IntPtr WindowHandle ) 
		{
			for(int i=0;i<items;i++) 
				if( menuitems[i].Equals(text) ) 
				{
					Form[] childs = mdiparent.MdiChildren;
					for(int x=0;x<items;x++) 
					{
						if(childs[x].Handle == WindowHandle) 
						{
							childs[x].MdiParent = mdiparent;
							childs[x].WindowState = System.Windows.Forms.FormWindowState.Maximized;
							childs[x].Activate();
							childs[x].Show();
							return;
						}
					}
				}

			if( items >= MAX_ELEMENTS ) {
				MessageBox.Show("Too many elements in EMPExtNav. Modify \"MAX_ELEMENTS\"!.", "Not allowed!");
				return;
			}

			menuitems[items] = text;
			item_start[items] = new System.Drawing.Point(0);
			item_end[items] = new System.Drawing.Point(0);
			handles[items] = WindowHandle;
			active_item = items++;
			this.Refresh();
		}

		private void ExtNav_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			int padding_left = 6;
			int i=0;	

			this.BackColor = this.CtlBackColor;
			this.Height = 30;

			e.Graphics.DrawLine( new Pen( new SolidBrush(System.Drawing.Color.LightGray ) ), 0, 0, this.Width, 0 );
			e.Graphics.DrawLine( new Pen( new SolidBrush(System.Drawing.Color.LightGray ) ), 0, 0, 0, this.Height );

			e.Graphics.DrawLine( new Pen( new SolidBrush(System.Drawing.Color.White) ), 0, this.Height-4, this.Width, this.Height-4 );
			e.Graphics.DrawLine( new Pen( new SolidBrush(this.ABackColor) ), 0, this.Height-3, this.Width, this.Height-3 );
			e.Graphics.DrawLine( new Pen( new SolidBrush(this.ABackColor) ), 0, this.Height-2, this.Width, this.Height-2 );
			e.Graphics.DrawLine( new Pen( new SolidBrush(this.ABackColor) ), 0, this.Height-1, this.Width, this.Height-1 );
			e.Graphics.DrawLine( new Pen( new SolidBrush(this.ABackColor) ), 0, this.Height, this.Width, this.Height );

			for(i=0;i<items;i++)
			{

				if(this.active_item==i) 
				{
					string measureString = menuitems[i];
					Font stringFont = new Font("Tahoma", 8, System.Drawing.FontStyle.Bold);

					SizeF layoutSize = new SizeF(500.0F, 30.0F);

					StringFormat newStringFormat = new StringFormat();

					int charactersFitted;
					int linesFilled;

					SizeF stringSize = new SizeF();
					stringSize = e.Graphics.MeasureString(
						measureString,
						stringFont,
						layoutSize,
						newStringFormat,
						out charactersFitted,
						out linesFilled
						);

					stringSize.Width += 10;
					stringSize.Height += 4;

					e.Graphics.DrawRectangle(
						new Pen(Color.White, 1),
						padding_left-1, this.Height-stringSize.Height-4, stringSize.Width, stringSize.Height
						);

					e.Graphics.FillRectangle( 
						new SolidBrush(this.ABackColor), padding_left, this.Height-stringSize.Height-3, stringSize.Width, stringSize.Height
						);

					e.Graphics.DrawLine(new Pen( System.Drawing.Brushes.Black, 1 ),
						new System.Drawing.Point( (int)stringSize.Width+padding_left+1, this.Height-(int)stringSize.Height-3 ),
						new System.Drawing.Point( (int)stringSize.Width+padding_left+1, this.Height-4 )
						);

					e.Graphics.DrawString(
						measureString,
						stringFont, new SolidBrush( this.AForeColor ),
						new PointF(padding_left+5, this.Height-stringSize.Height),
						newStringFormat
						);

					item_start[i] = new Point( (int)padding_left, (int)this.Height-(int)stringSize.Height-3 );
					item_end[i] = new Point( (int)padding_left+(int)stringSize.Width, (int)this.Height );

					padding_left += (int)stringSize.Width+3;
				} 
				else if( hover_item == i ) 
				{
					string measureString = menuitems[i];
					Font stringFont = new Font("Tahoma", 8, System.Drawing.FontStyle.Regular);

					SizeF layoutSize = new SizeF(500.0F, 30.0F);

					StringFormat newStringFormat = new StringFormat();

					int charactersFitted;
					int linesFilled;

					SizeF stringSize = new SizeF();
					stringSize = e.Graphics.MeasureString(
						measureString,
						stringFont,
						layoutSize,
						newStringFormat,
						out charactersFitted,
						out linesFilled
						);

					stringSize.Width += 10;
					stringSize.Height += 4;

					e.Graphics.DrawRectangle(
						new Pen(Color.White, 1),
						padding_left-1, this.Height-stringSize.Height-4, stringSize.Width, stringSize.Height
						);

					e.Graphics.FillRectangle( 
						new SolidBrush(System.Drawing.Color.Beige), padding_left, this.Height-stringSize.Height-3, stringSize.Width, stringSize.Height
						);

					e.Graphics.DrawLine(new Pen( System.Drawing.Brushes.Black, 1 ),
						new System.Drawing.Point( (int)stringSize.Width+padding_left+1, this.Height-(int)stringSize.Height-3 ),
						new System.Drawing.Point( (int)stringSize.Width+padding_left+1, this.Height-4 )
						);

					e.Graphics.DrawString(
						measureString,
						stringFont, new SolidBrush( this.AForeColor ),
						new PointF(padding_left+5, this.Height-stringSize.Height),
						newStringFormat
						);

					item_start[i] = new Point( (int)padding_left, (int)this.Height-(int)stringSize.Height-3 );
					item_end[i] = new Point( (int)padding_left+(int)stringSize.Width, (int)this.Height );

					padding_left += (int)stringSize.Width+3;
				}
				else 
				{
					string measureString = menuitems[i];
					Font stringFont = new Font("Tahoma", 8, System.Drawing.FontStyle.Regular);

					SizeF layoutSize = new SizeF(500.0F, 30.0F);

					StringFormat newStringFormat = new StringFormat();

					int charactersFitted;
					int linesFilled;

					SizeF stringSize = new SizeF();
					stringSize = e.Graphics.MeasureString(
						measureString,
						stringFont,
						layoutSize,
						newStringFormat,
						out charactersFitted,
						out linesFilled
						);

					stringSize.Width += 10;
					stringSize.Height += 4;

					e.Graphics.FillRectangle( 
						new SolidBrush(this.IBackColor), padding_left, this.Height-stringSize.Height-3, stringSize.Width, stringSize.Height
						);

					if( this.active_item != i+1 ) 
					{
						e.Graphics.DrawLine(new Pen( new SolidBrush( this.IForeColor ), 1 ),
							new System.Drawing.Point( (int)stringSize.Width+padding_left+1, this.Height-(int)stringSize.Height-3 ),
							new System.Drawing.Point( (int)stringSize.Width+padding_left+1, this.Height-6 )
							);
					}

					e.Graphics.DrawString(
						measureString,
						stringFont, new SolidBrush( this.IForeColor ),
						new PointF(padding_left+5, this.Height-stringSize.Height),
						newStringFormat
						);

					item_start[i] = new Point( (int)padding_left, (int)this.Height-(int)stringSize.Height-3 );
					item_end[i] = new Point( (int)padding_left+(int)stringSize.Width, (int)this.Height );
					padding_left += (int)stringSize.Width+3;
				}
			}

			e.Graphics.Save();
			e.Graphics.Flush();
		}

		private void ExtNav_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			bool need_activation;
			for(int i=0;i<items;i++) 
			{
				if(	(e.X > item_start[i].X && e.Y > item_start[i].Y) &&
					(e.X < item_end[i].X  &&  e.Y < item_end[i].Y)
					)
				{
					this.Cursor = System.Windows.Forms.Cursors.Hand;
					need_activation = (hover_item!=i);
					hover_item = i;
					if(need_activation) this.Refresh();
					return;
				}
			}
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		private bool is_active_handle( System.IntPtr handle) 
		{
			for(int i=0;i<items;i++) if( handles[i] == handle && active_item == i ) return true;
			return false;
		}

		private void ExtNav_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			active_item = hover_item;
			Form[] childs = mdiparent.MdiChildren;
			for(int i=0;i<items;i++) 
			{
				if( (e.X > item_start[i].X && e.Y > item_start[i].Y) &&
					(e.X < item_end[i].X  &&  e.Y < item_end[i].Y)
					)
				{
					childs[i].Activate();
					active_item = i;
					this.Refresh();
					return;
				}
			}
		}

		public bool delete( System.IntPtr WindowHandle ) 
		{
			int x=0;
			bool deleted = false;

			string[] x_menuitems = new string[MAX_ELEMENTS];
			System.Drawing.Point[] x_item_start = new System.Drawing.Point[MAX_ELEMENTS];
			System.Drawing.Point[] x_item_end = new System.Drawing.Point[MAX_ELEMENTS];
			System.IntPtr[] x_handles = new System.IntPtr[MAX_ELEMENTS];

			for(int i=0;i<items;i++) 
			{
				if( !handles[i].Equals(WindowHandle) ) 
				{
					x_handles[x] = handles[i];
					x_menuitems[x] = menuitems[i];
					x_item_start[x] = item_start[i];
					x_item_end[x] = item_end[i];
					x++;
				} 
				else 
				{
					if(--active_item < 0) active_item = 0;
					deleted = true;
				}
			}

			items = x;
			handles = x_handles;
			menuitems = x_menuitems;
			item_start = x_item_start;
			item_end = x_item_end;

			this.Refresh();
			return deleted;
		}

		public void sync( System.IntPtr WindowHandle ) 
		{
			for(int i=0;i<items;i++) 
			{
				if( handles[i].Equals(WindowHandle) ) 
				{
					active_item = i;
					hover_item = -1;
					this.Refresh();
					break;
				}
			}
		}

		private void ExtNav_MouseLeave(object sender, System.EventArgs e)
		{
			hover_item = -1;
			this.Refresh();
		}

	}
}
